## This file generated by InlineX::C2XS (version 0.28) using Inline::C (version 0.82)
package Math::GMPf::V;
use strict;
use warnings;

require Exporter;
*import = \&Exporter::import;
require DynaLoader;

our $VERSION = '0.54';
$VERSION = eval $VERSION;
DynaLoader::bootstrap Math::GMPf::V $VERSION;

@Math::GMPf::V::EXPORT = ();
@Math::GMPf::V::EXPORT_OK = ();

sub _buggy {
  return 0 unless $^O =~ /MSWin/;
  my $cc = ___GMP_CC();		# __GMP_CC
  my $cflags = ___GMP_CFLAGS();	# __GMP_CFLAGS

  return 0 if ( defined($cc)     && $cc     =~/\-D__USE_MINGW_ANSI_STDIO/ );
  return 0 if ( defined($cflags) && $cflags =~/\-D__USE_MINGW_ANSI_STDIO/ );
  return 1;
}

sub dl_load_flags {0} # Prevent DynaLoader from complaining and croaking

1;
