
#ifndef MATH_GMPF_UNUSED_H
#define MATH_GMPF_UNUSED_H 1

#define PERL_UNUSED_ARG2(a,b) PERL_UNUSED_ARG(a);PERL_UNUSED_ARG(b);

#define PERL_UNUSED_ARG3(a,b,c) PERL_UNUSED_ARG(a);PERL_UNUSED_ARG(b);\
                                PERL_UNUSED_ARG(c);

#define PERL_UNUSED_ARG4(a,b,c,d) PERL_UNUSED_ARG(a);PERL_UNUSED_ARG(b);\
                                  PERL_UNUSED_ARG(c);PERL_UNUSED_ARG(d);

#endif

